@@ -41,35 +41,37 @@ typedef struct W_Button {
 	WMAlignment alignment:2;
 
 	unsigned int selected:1;
-	
+
 	unsigned int enabled:1;
 
+        unsigned int dimsWhenDisabled:1;
+
 	unsigned int bordered:1;
 
 	unsigned int springLoaded:1;
-	
+
 	unsigned int pushIn:1;	       /* change relief while pushed */
-	
+
 	unsigned int pushLight:1;      /* highlight while pushed */
-	
+
 	unsigned int pushChange:1;     /* change caption while pushed */
-	    
+
 	unsigned int stateLight:1;     /* state indicated by highlight */
-	
+
 	unsigned int stateChange:1;    /* state indicated by caption change */
 
 	unsigned int statePush:1;      /* state indicated by relief */
 
 	unsigned int continuous:1;     /* continually perform action */
-	/* */
+
 	unsigned int prevSelected:1;
 
 	unsigned int pushed:1;
-	
+
 	unsigned int wasPushed:1;
 
 	unsigned int redrawPending:1;
-	
+
 	unsigned int addedObserver:1;
     } flags;
 } Button;
@@ -135,13 +137,13 @@ WMCreateCustomButton(WMWidget *parent, int behaviourMask)
     bPtr->flags.stateLight = (behaviourMask & WBBStateLightMask)!=0;
     bPtr->flags.stateChange = (behaviourMask & WBBStateChangeMask)!=0;
     bPtr->flags.statePush = (behaviourMask & WBBStatePushMask)!=0;
-    
+
     W_ResizeView(bPtr->view, DEFAULT_BUTTON_WIDTH, DEFAULT_BUTTON_HEIGHT);
     bPtr->flags.alignment = DEFAULT_BUTTON_ALIGNMENT;
     bPtr->flags.bordered = DEFAULT_BUTTON_IS_BORDERED;
-    
-    bPtr->flags.enabled = 1;
 
+    bPtr->flags.enabled = 1;
+    bPtr->flags.dimsWhenDisabled = 1;
 
     WMCreateEventHandler(bPtr->view, ExposureMask|StructureNotifyMask,
 			 handleEvents, bPtr);
@@ -239,31 +241,38 @@ updateDisabledMask(WMButton *bPtr)
     if (bPtr->image) {
 	XGCValues gcv;
 
-	bPtr->dimage->mask = XCreatePixmap(dpy, scr->stipple,
-					   bPtr->dimage->width, 
-					   bPtr->dimage->height, 1);
+        if (bPtr->dimage->mask) {
+            XFreePixmap(dpy, bPtr->dimage->mask);
+            bPtr->dimage->mask = None;
+        }
 
-	XSetForeground(dpy, scr->monoGC, 0);
-	XFillRectangle(dpy, bPtr->dimage->mask, scr->monoGC, 0, 0,
-		       bPtr->dimage->width, bPtr->dimage->height);
+        if (bPtr->flags.dimsWhenDisabled) {
+            bPtr->dimage->mask = XCreatePixmap(dpy, scr->stipple,
+                                               bPtr->dimage->width,
+                                               bPtr->dimage->height, 1);
 
-	gcv.foreground = 1;
-	gcv.background = 0;
-	gcv.stipple = scr->stipple;
-	gcv.fill_style = FillStippled;
-	gcv.clip_mask = bPtr->image->mask;
-	gcv.clip_x_origin = 0;
-	gcv.clip_y_origin = 0;
-	
-	XChangeGC(dpy, scr->monoGC, GCForeground|GCBackground|GCStipple
-		  |GCFillStyle|GCClipMask|GCClipXOrigin|GCClipYOrigin, &gcv);
+            XSetForeground(dpy, scr->monoGC, 0);
+            XFillRectangle(dpy, bPtr->dimage->mask, scr->monoGC, 0, 0,
+                           bPtr->dimage->width, bPtr->dimage->height);
 
-	XFillRectangle(dpy, bPtr->dimage->mask, scr->monoGC, 0, 0,
-		       bPtr->dimage->width, bPtr->dimage->height);
+            gcv.foreground = 1;
+            gcv.background = 0;
+            gcv.stipple = scr->stipple;
+            gcv.fill_style = FillStippled;
+            gcv.clip_mask = bPtr->image->mask;
+            gcv.clip_x_origin = 0;
+            gcv.clip_y_origin = 0;
 
-	gcv.fill_style = FillSolid;
-	gcv.clip_mask = None;
-	XChangeGC(dpy, scr->monoGC, GCFillStyle|GCClipMask, &gcv);
+            XChangeGC(dpy, scr->monoGC, GCForeground|GCBackground|GCStipple
+                      |GCFillStyle|GCClipMask|GCClipXOrigin|GCClipYOrigin, &gcv);
+
+            XFillRectangle(dpy, bPtr->dimage->mask, scr->monoGC, 0, 0,
+                           bPtr->dimage->width, bPtr->dimage->height);
+
+            gcv.fill_style = FillSolid;
+            gcv.clip_mask = None;
+            XChangeGC(dpy, scr->monoGC, GCFillStyle|GCClipMask, &gcv);
+        }
     }
 }
 
@@ -410,7 +419,7 @@ WMSetButtonDisabledTextColor(WMButton *bPtr, WMColor *color)
 void
 WMSetButtonSelected(WMButton *bPtr, int isSelected)
 {
-    bPtr->flags.selected = isSelected;
+    bPtr->flags.selected = isSelected ? 1 : 0;
  
     if (bPtr->view->flags.realized) {
 	paintButton(bPtr);
@@ -453,7 +462,7 @@ WMSetButtonFont(WMButton *bPtr, WMFont *font)
 void
 WMSetButtonEnabled(WMButton *bPtr, Bool flag)
 {
-    bPtr->flags.enabled = flag;
+    bPtr->flags.enabled = ((flag==0) ? 0 : 1);
 
     if (bPtr->view->flags.mapped) {
 	paintButton(bPtr);
@@ -461,19 +470,36 @@ WMSetButtonEnabled(WMButton *bPtr, Bool flag)
 }
 
 
+int
+WMGetButtonEnabled(WMButton *bPtr)
+{
+    CHECK_CLASS(bPtr, WC_Button);
+
+    return bPtr->flags.enabled;
+}
+
+
+void
+WMSetButtonImageDimsWhenDisabled(WMButton *bPtr, Bool flag)
+{
+    bPtr->flags.dimsWhenDisabled = ((flag==0) ? 0 : 1);
+
+    updateDisabledMask(bPtr);
+}
+
+
 void
 WMSetButtonTag(WMButton *bPtr, int tag)
 {
     bPtr->tag = tag;
 }
 
-    
 
 void
 WMPerformButtonClick(WMButton *bPtr)
 {
     CHECK_CLASS(bPtr, WC_Button);
-  
+
     if (!bPtr->flags.enabled)
 	return;
 
@@ -560,7 +586,7 @@ WMGroupButtons(WMButton *bPtr, WMButton *newMember)
 void
 WMSetButtonContinuous(WMButton *bPtr, Bool flag)
 {
-    bPtr->flags.continuous = flag;
+    bPtr->flags.continuous = ((flag==0) ? 0 : 1);
     if (bPtr->timer) {
 	WMDeleteTimerHandler(bPtr->timer);
 	bPtr->timer = NULL;
@@ -650,7 +676,7 @@ paintButton(Button *bPtr)
 	}
     }
 
-    W_PaintTextAndImage(bPtr->view, False, WMColorGC(textColor),
+    W_PaintTextAndImage(bPtr->view, True, WMColorGC(textColor),
 			(bPtr->font!=NULL ? bPtr->font : scrPtr->normalFont),
 			relief, caption, bPtr->flags.alignment, image, 
 			bPtr->flags.imagePosition, gc, offset);
@@ -750,7 +776,7 @@ handleActionEvents(XEvent *event, void *data)
 	if (event->xbutton.button == Button1) {
 	    if (bPtr->flags.pushed) {
 		if (bPtr->groupIndex==0 || 
-		    bPtr->flags.selected && bPtr->groupIndex > 0)
+		    (bPtr->flags.selected && bPtr->groupIndex > 0))
 		    doclick = 1;
 		dopaint = 1;
 		if (bPtr->flags.springLoaded) {
