@@ -1,8 +1,10 @@
 
 
 #include "WINGsP.h"
+#include "wconfig.h"
 
 #include <X11/Xutil.h>
+#include <X11/Xatom.h>
 #include <X11/keysym.h>
 #include <X11/cursorfont.h>
 
@@ -300,7 +302,7 @@ static char *CHECK_MARK[] = {
 #define STIPPLE_WIDTH 8
 #define STIPPLE_HEIGHT 8
 static unsigned char STIPPLE_BITS[] = {
-    0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa
+    0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55
 };
 
 
@@ -410,7 +412,7 @@ loadPixmaps(WMScreen *scr)
     if (!image)
 	image = RLoadImage(scr->rcontext, X_WINGS_IMAGES_FILE, 0);
     if (!image) {
-	wwarning("WINGs: could not load widget images file: %s", 
+	wwarning(_("WINGs: could not load widget images file: %s"), 
 		 RMessageForError(RErrorCode));
 	return False;
     }
@@ -419,85 +421,85 @@ loadPixmaps(WMScreen *scr)
     tmp = RGetSubImage(image, 0, 0, 24, 24);
     RCombineImageWithColor(tmp, &gray);
     scr->homeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* make it have a white background */
     tmp = RGetSubImage(image, 0, 0, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altHomeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
 
     /* trash can */
     tmp = RGetSubImage(image, 104, 0, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->trashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     tmp = RGetSubImage(image, 104, 0, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altTrashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* create dir */
     tmp = RGetSubImage(image, 104, 24, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->createDirIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     tmp = RGetSubImage(image, 104, 24, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altCreateDirIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* diskettes */
     tmp = RGetSubImage(image, 24, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->disketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     tmp = RGetSubImage(image, 24, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altDisketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* unmount */
     tmp = RGetSubImage(image, 0, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->unmountIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     tmp = RGetSubImage(image, 0, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altUnmountIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
 
     /* Magnifying Glass Icon for ColorPanel */
     tmp = RGetSubImage(image, 24, 0, 40, 32);
     RCombineImageWithColor(tmp, &gray);
     scr->magnifyIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* ColorWheel Icon for ColorPanel */
     tmp = RGetSubImage(image, 0, 25, 24, 24);
     scr->wheelIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* GrayScale Icon for ColorPanel */
     tmp = RGetSubImage(image, 65, 0, 40, 24);
     scr->grayIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* RGB Icon for ColorPanel */
     tmp = RGetSubImage(image, 25, 33, 40, 24);
     scr->rgbIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* CMYK Icon for ColorPanel */
     tmp = RGetSubImage(image, 65, 25, 40, 24);
     scr->cmykIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* HSB Icon for ColorPanel */
     tmp = RGetSubImage(image, 0, 57, 40, 24);
     scr->hsbIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* CustomColorPalette Icon for ColorPanel */
     tmp = RGetSubImage(image, 81, 57, 40, 24);
     scr->customPaletteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     /* ColorList Icon for ColorPanel */
     tmp = RGetSubImage(image, 41, 57, 40, 24);
     scr->colorListIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
+    RReleaseImage(tmp);
     
-    RDestroyImage(image);
+    RReleaseImage(image);
 
 #if 0
     scr->defaultObjectIcon =
@@ -516,13 +518,13 @@ loadPixmaps(WMScreen *scr)
 
 
 WMScreen*
-WMOpenScreen()
+WMOpenScreen(const char *display)
 {
-    Display *dpy = XOpenDisplay("");
+    Display *dpy = XOpenDisplay(display);
     
     if (!dpy) {
 	wwarning("WINGs: could not open display %s",
-		 XDisplayName(""));
+		 XDisplayName(display));
 	return NULL;
     }
     
@@ -583,7 +585,7 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 	    "WM_STATE"
     };
     Atom atoms[sizeof(atomNames)/sizeof(char*)];
-    int i = 0;
+    int i;
 
     if (!initialized) {
 
@@ -616,6 +618,51 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 
     scrPtr->fontCache = WMCreateHashTable(WMStringPointerHashCallbacks);
 
+    scrPtr->fontSetCache = WMCreateHashTable(WMStringPointerHashCallbacks);
+
+
+    /* create input method stuff */
+    W_InitIMStuff(scrPtr);
+
+    /* Create missing CUT_BUFFERs */
+    {
+        Atom *rootWinProps;
+        int exists[8] = {0, 0, 0, 0, 0, 0, 0, 0};
+        int count;
+
+        rootWinProps = XListProperties(display, scrPtr->rootWin, &count);
+        for (i=0; i<count; i++) {
+            switch(rootWinProps[i]) {
+            case XA_CUT_BUFFER0:
+                exists[0] = 1; break;
+            case XA_CUT_BUFFER1:
+                exists[1] = 1; break;
+            case XA_CUT_BUFFER2:
+                exists[2] = 1; break;
+            case XA_CUT_BUFFER3:
+                exists[3] = 1; break;
+            case XA_CUT_BUFFER4:
+                exists[4] = 1; break;
+            case XA_CUT_BUFFER5:
+                exists[5] = 1; break;
+            case XA_CUT_BUFFER6:
+                exists[6] = 1; break;
+            case XA_CUT_BUFFER7:
+                exists[7] = 1; break;
+            default:
+                break;
+            }
+        }
+        if (rootWinProps) {
+            XFree(rootWinProps);
+        }
+        for (i=0; i<8; i++) {
+            if (!exists[i]) {
+                XStoreBuffer(display, "", 0, i);
+            }
+        }
+    }
+
     scrPtr->ignoredModifierMask = 0;
     {
 	int i;
@@ -713,8 +760,8 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 	scrPtr->boldFont = scrPtr->normalFont;
 
     if (!scrPtr->normalFont) {
-	wwarning("could not load any fonts. Make sure your font installation"
-		 "and locale settings are correct.");
+	wwarning(_("could not load any fonts. Make sure your font installation"
+                   " and locale settings are correct."));
 
 	return NULL;
     }
@@ -816,11 +863,8 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     XInternAtoms(display, atomNames, sizeof(atomNames)/sizeof(char*), False,
 		 atoms);
 #else
-    {
-	int i;
-	for (i = 0; i < sizeof(atomNames)/sizeof(char*); i++) {
-	    atoms[i] = XInternAtom(display, atomNames[i], False);
-	}
+    for (i = 0; i < sizeof(atomNames)/sizeof(char*); i++) {
+        atoms[i] = XInternAtom(display, atomNames[i], False);
     }
 #endif
 
@@ -852,15 +896,14 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     scrPtr->wmIconDragOffsetAtom = atoms[i++];
 
     scrPtr->wmStateAtom = atoms[i++];
-	
-    scrPtr->rootView = W_CreateRootView(scrPtr);
 
+    scrPtr->rootView = W_CreateRootView(scrPtr);
 
     scrPtr->balloon = W_CreateBalloon(scrPtr);
 
 
     W_InitApplication(scrPtr);
-    
+
     return scrPtr;
 }
 
@@ -934,7 +977,7 @@ WMMapWidget(WMWidget *w)
 void
 WMReparentWidget(WMWidget *w, WMWidget *newParent, int x, int y)
 {
-    W_ReparentView(W_VIEW(w), newParent, x, y);
+    W_ReparentView(W_VIEW(w), W_VIEW(newParent), x, y);
 }
 
 
