@@ -1,7 +1,7 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,11 @@
 #include "window.h"
 #include "funcs.h"
 
+
+#define TILE_NORMAL	0
+#define TILE_CLIP	1
+
+
 typedef struct WIcon {
     WCoreWindow 	*core;
     WWindow 		*owner;	       /* owner window */
@@ -37,11 +42,12 @@ typedef struct WIcon {
     char		*file;	       /* the file with the icon image */
     RImage 		*image;
 
+    unsigned int 	tile_type:4;
     unsigned int 	show_title:1;
     unsigned int 	has_titlebar:1;
     unsigned int 	force_paint:1; /* True for icon update and repaint */
     unsigned int 	selected:1;
-    unsigned int	step:1;
+    unsigned int	step:3;	       /* selection cycle step */
     unsigned int	shadowed:1;    /* If the icon is to be blured */
     unsigned int 	mapped:1;
     unsigned int	highlighted:1;
@@ -52,7 +58,7 @@ typedef struct WIcon {
 					* color */
 } WIcon;
 
-WIcon *wIconCreateWithIconFile(WScreen *scr, char *iconfile);
+WIcon *wIconCreateWithIconFile(WScreen *scr, char *iconfile, int tile);
 WIcon *wIconCreate(WWindow *wwin);
 void wIconDestroy(WIcon *icon);
 void wIconPaint(WIcon *icon);
@@ -64,4 +70,6 @@ void wIconSelect(WIcon *icon);
 
 RImage *wIconValidateIconSize(WScreen *scr, RImage *icon);
 
+char *wIconStore(WIcon *icon);
+
 #endif /* WMICON_H_ */
